
#include  "badge_init.h"
#include  "badge_isr.h"
#include  "badge_leds.h"
#include  "badge_basics.h"
#include  "badge_flash.h"
#include  "badge_lcd.h"
#include  "badge_iru.h"
#include  "badge_rf.h"

#define   MY_IR_ID    0x50
#define   MY_RF_ID_H  0x50
#define   MY_RF_ID_L  0x50
#define   FR_RF_ID_H  0x60
#define   FR_RF_ID_L  0x60

unsigned char setting_counter;
unsigned int led1,led2,led3,led4,led5;

void main(void)
{

  unsigned short inte = 0;
  unsigned short dir = 0;
  unsigned char ticker,ticket,new_ticket,ticker_ir;
  unsigned char b_led1,b_led2,b_led3,b_led4;
  unsigned char pRFData[5];
  unsigned int i;
  unsigned char pid, dimming_led; 
  
  setting_counter = 1;

  led1 = 1;
  led2 = 0;
  led3 = 0;
  led4 = 0;
  led5 = 0;

  b_led1 = 0;
  b_led2 = 0;
  b_led3 = 0;
  b_led4 = 0;

  badge_init_osc();
  badge_init_io();
  badge_init_spi();
  badge_init_uart();
  badge_init_pwm();

  Wait(100);                       // Power Up Wait

  badge_led_grid_init();

  _EINT();

  badge_led_grid_draw_screen(led1,led2,led3,led4,led5);

  set_rf_settings(MY_RF_ID_H,MY_RF_ID_L,0x10,0x05);

 
  while(TRUE) {
    Wait(50);

    get_rf_status();

  if (badge_poll_switch() == SWITCH_UP){
    pRFData[0] = 0xD1;
    send_rf_packet(0x01,FR_RF_ID_H,FR_RF_ID_L,0x01,pRFData); 
    while (badge_poll_switch() == SWITCH_UP);
  } else if (badge_poll_switch() == SWITCH_DOWN) {
    pRFData[0] = 0xD2;
    send_rf_packet(0x01,FR_RF_ID_H,FR_RF_ID_L,0x01,pRFData); 
    while (badge_poll_switch() == SWITCH_DOWN);
  } else if (badge_poll_switch() == SWITCH_IN) {
    pRFData[0] = 0xD3;
    send_rf_packet(0x01,FR_RF_ID_H,FR_RF_ID_L,0x01,pRFData); 
    while (badge_poll_switch() == SWITCH_IN);
  }



  ticket = badge_iru_status();
  if (ticket & UART_RX_STATUS) {
    // UART BYTE RECEIVED HANDLE
  } else if (ticket & IR_RX_STATUS) {
    // IR BYTE RECEIVED HANDLE
    ticket = badge_iru_get_ir_byte();
    if (ticket != MY_IR_ID) {
      if (new_ticket == 0) {
        dir = 0;
        ticker_ir = 0;
        b_led1 = 0;
        b_led2 = 0;
        b_led3 = 0;
        b_led4 = 0;
      }  
        ticker_ir = 0;
        new_ticket = 1;
    }
  }

  if(++ticker == 10) {
    badge_iru_send_ir_byte(MY_IR_ID);
    ticker = 0;
   }

  if (new_ticket == 1) {
    if (b_led1 <= 0x32){
      badge_single_led(BLED1,b_led1++);
      badge_single_led(BLED2,b_led2++);
      badge_single_led(BLED3,b_led3++);
      badge_single_led(BLED4,b_led4++);
    }        
  }

  if(++ticker_ir == 20) {
      new_ticket = 0;
      dimming_led = 1;
   } 

  if(b_led1 && dimming_led){
      badge_single_led(BLED1,--b_led1);
      badge_single_led(BLED2,--b_led2);
      badge_single_led(BLED3,--b_led3);
      badge_single_led(BLED4,--b_led4);
  } else {
    dimming_led = 0;
  }


  }
}

void usart1_rx (void) __interrupt[UART1RX_VECTOR] 
{
  unsigned char ret_val;
  unsigned char status;
  unsigned char newPacket[DATA_BUFFER_LENGTH];

  ret_val = handle_rf_byte(RXBUF1, &status, newPacket);
  if(ret_val == NEW_STATUS) {
    if ( status & 0x01) {
      Wait(50);
      get_rf_packet();
    } 
  } else if (ret_val == NEW_RX_PACKET) {
    // RECEIVED NEW PACKET IN BUFFER
    if (newPacket[4]==0xD1) {
      if (led1 > 0) {
        led1 = led1 << 1;
        if (led1 == 0x200) {
          led1 = 0;
          led2 = 1;
        }
      } else if (led2 > 0) {
        led2 = led2 << 1;
        if (led2 == 0x200) {
          led2 = 0;
          led3 = 1;
        }
      } else if (led3 > 0) {
        led3 = led3 << 1;
        if (led3 == 0x200) {
          led3 = 0;
          led4 = 1;
        }
      } else if (led4 > 0) {
        led4 = led4 << 1;
        if (led4 == 0x200) {
          led4 = 0;
          led5 = 1;
        }
     } else if (led5 > 0) {
        led5 = led5 << 1;
        if (led5 == 0x200) {
          led5 = 0;
          led1 = 1;
        }
     }
     badge_led_grid_draw_screen(led1,led2,led3,led4,led5);
    }
    if (newPacket[4]==0xD3) {
      if (get_motor_status()) badge_motor(FALSE);
      else badge_motor(TRUE);
    }
    if (newPacket[4]==0xD2) {
      if (led1 > 0) {
        led1 = led1 >> 1;
        if ((led1 & 0x00FF)==0) {
          led1 = 0;
          led5 = 0x100;
        }
      } else if (led2 > 0) {
        led2 = led2 >> 1;
        if ((led2 & 0x00FF)==0) {
          led2 = 0;
          led1 = 0x100;
        }
      } else if (led3 > 0) {
        led3 = led3 >> 1;
        if ((led3 & 0x00FF)==0) {
          led3 = 0;
          led2 = 0x100;
        }
      } else if (led4 > 0) {
        led4 = led4 >> 1;
        if ((led4 & 0x00FF)==0) {
          led4 = 0;
          led3 = 0x100;
        }
     } else if (led5 > 0) {
        led5 = led5 >> 1;
        if ((led5 & 0x00FF)==0) {
          led5 = 0;
          led4 = 0x100;
        }
     }
     badge_led_grid_draw_screen(led1,led2,led3,led4,led5);
    }
  }
}


