#include  "badge_alpha.h"

#include  "badge_init.h"
#include  "badge_isr.h"
#include  "badge_leds.h"
#include  "badge_basics.h"
#include  "badge_flash.h"
#include  "badge_lcd.h"
#include  "badge_iru.h"
#include  "badge_rf.h"

#include <stdlib.h>

#define   MY_BADGE_ID    0x86
#define   TIMER_SCALER  50
#define   BRIDGE_BLINK  60
#define   TEAM_TIMEOUT  500

#define   META_1_1    0x1F0
#define   META_1_2    0x1E0
#define   META_1_3    0x1C0
#define   META_1_4    0x180
#define   META_1_5    0x100
#define   META_2_1    0x00E
#define   META_2_2    0x01C
#define   META_2_3    0x038
#define   META_2_4    0x070
#define   META_2_5    0x0E0
#define   META_3_1    0x001
#define   META_3_2    0x003
#define   META_3_3    0x007
#define   META_3_4    0x00F
#define   META_3_5    0x01F

#define   META_STATE    0
#define   BRIDGE_STATE  1
#define   MSG_STATE     2
#define   HEAD_STATE    3
#define   MELODY_STATE  4
#define   FINISH_STATE  5

#define   F_TAG       0x01000     /* Flash address for state */
#define   F_STATE     0x01001     /* Flash address for state */
#define   F_META      0x01002     /* Flash address for Meta */

unsigned int tmr_counter;
unsigned char comm_toggle;
unsigned char pRFData[5];
unsigned int led1,led2,led3,led4,led5;
unsigned char bridge_disp;
unsigned char bridge_toggle, bridge_tick, bridge_pos, bridge_cols;
unsigned int bridge_loops;
unsigned char bridge_deck[5];
unsigned char msg_toggle, msg_tick, msg_pos, msg_cols;
unsigned char msg_deck[5];

unsigned char state_ctrl;
unsigned char meta_ctrl;

unsigned long team_ticks[50];
unsigned char team_new[50];
unsigned char old_state;
unsigned char digit_10, digit_1,msg_meta;

unsigned char simon_tags;
unsigned char simon_says[16];
unsigned char simon_index;
unsigned char simon_in;
unsigned char simon_echo;
unsigned char simon_once;

unsigned char head_deck[5];
unsigned char head_toggle, head_tick, head_pos, head_cols;
unsigned char fin_deck[5];
unsigned char fin_toggle, fin_tick, fin_pos, fin_cols;

unsigned int sine_index;
unsigned char volume;
unsigned char simon_speaks;
unsigned int simon_ticks;
unsigned long simon_echo_ticks;
unsigned int melody_ticks;
unsigned char melody_pos;
unsigned char motor_mel;

unsigned char *f_ptr;

unsigned long meta_ticker;
const unsigned char SIMON_MELODY[] = {3,3,4,5,5,4,3,2,1,1,2,3,2,1,1};
const unsigned int TONES[] = {(CLOCK_RATE/22)-1,(CLOCK_RATE/25)-1,(CLOCK_RATE/28)-1,(CLOCK_RATE/30)-1,(CLOCK_RATE/33)-1};
const unsigned char BRIDGE_MESSAGE[] = {23,1,12,11,0,15,22,5,18,0,20,8,5,0,13,1,14,8,1,20,20,1,14,0,2,18,9,4,7,5,0,0};
const unsigned char MSG_MESSAGE[] = {20,5,1,13,0};
const unsigned char HEAD_MESSAGE[] = {8,5,1,4,16,8,15,14,5,19,0,0};
const unsigned char FINISH_MESSAGE[] = {2,18,15,15,11,12,25,14,0,9,3,5,0,3,18,5,1,13,0,6,1,3,20,15,18,25,27,15,12,4,0,6,21,12,20,15,14,0,16,9,5,18,0,0};

bool WriteFlashByte(unsigned char *Addr, unsigned char Data);
void Flash_clr( unsigned char *Data_ptr );

void main(void)
{

  unsigned short inte = 0;
  unsigned short dir = 0;
  unsigned char ticker,ticket,new_ticket,ticker_ir;
  unsigned char b_led1,b_led2,b_led3,b_led4;
  unsigned int i;
  unsigned char pid, dimming_led; 

  
  tmr_counter = 0;
  comm_toggle = 0;
  bridge_toggle = 0;
  bridge_disp = 0;
  bridge_loops = 0;
  bridge_tick = BRIDGE_BLINK;
  head_tick = BRIDGE_BLINK;
  head_pos = 0;
  head_cols = 0;
  head_toggle = 0;
  meta_ticker = 0;
  sine_index = 0;
  volume = 1;
  simon_ticks = 0;
  simon_speaks = 0;
  melody_pos = 0;
  melody_ticks = 0;
  motor_mel = 0;

  b_led1 = 0;
  b_led2 = 0;
  b_led3 = 0;
  b_led4 = 0;

  badge_init_osc();
  badge_init_io();
  badge_init_spi();
  badge_init_uart();
  badge_init_pwm();

  f_ptr = (unsigned char *)F_TAG;
  if(*f_ptr == 0x6D) {
    f_ptr++;
    state_ctrl = *f_ptr;
    f_ptr++;
    meta_ctrl = *f_ptr;
  } else {
    state_ctrl = META_STATE;
    meta_ctrl = 0;
    f_ptr = (unsigned char *)F_TAG;
    Flash_clr(f_ptr);
    WriteFlashByte(f_ptr++, 0x6D);
    WriteFlashByte(f_ptr++, state_ctrl);
    WriteFlashByte(f_ptr++, meta_ctrl);
  }


  for(i=0;i<50;i++) {
    team_ticks[i] = TEAM_TIMEOUT;
    team_new[i] = 0;
  }

  Wait(500);                       // Power Up Wait

  badge_led_grid_init();
  badge_led_grid_draw_screen(0,0,0,0,0);

  _EINT();

  set_rf_settings(0x00,MY_BADGE_ID,0x10,0x03);
  Wait(250);
  set_rf_settings(0x00,MY_BADGE_ID,0x10,0x03);
  Wait(250);

  TACTL = TASSEL1 + TACLR + TAIE;       // SMCLK, clr. TAR, interrupt
  TACTL |= MC1;                         // Start Timer_a in continous

 
  while(TRUE) {


    Wait(100);
    
    if (state_ctrl != MELODY_STATE) {
      badge_motor(FALSE);
    } else {
      if(++motor_mel > 10) {
      motor_mel = 0;
      badge_motor(FALSE);
      }
    }

    get_rf_status();

  if (badge_poll_switch() == SWITCH_UP){
    volume = 2;
    while (badge_poll_switch() == SWITCH_UP);
  } else if (badge_poll_switch() == SWITCH_DOWN) {
    volume = 1;
    while (badge_poll_switch() == SWITCH_DOWN);
  } else if (badge_poll_switch() == SWITCH_IN) {
    while (badge_poll_switch() == SWITCH_IN);
  }



  ticket = badge_iru_status();
  if (ticket & UART_RX_STATUS) {
    // UART BYTE RECEIVED HANDLE
  } else if (ticket & IR_RX_STATUS) {
    // IR BYTE RECEIVED HANDLE
    ticket = badge_iru_get_ir_byte();
    if (ticket != MY_BADGE_ID) {
      if (ticket == 0xCD) {
          state_ctrl = META_STATE;
          meta_ctrl = 0;
          led1 = 0;
          led2 = 0;
          led3 = 0;
          led4 = 0;
          led5 = 0;
          badge_led_grid_draw_screen(0,0,0,0,0);
          f_ptr = (unsigned char *)F_TAG;
          Flash_clr(f_ptr);
          WriteFlashByte(f_ptr++, 0x6D);
          WriteFlashByte(f_ptr++, state_ctrl);
          WriteFlashByte(f_ptr++, meta_ctrl);
          tmr_counter = 0;
          comm_toggle = 0;
          bridge_toggle = 0;
          bridge_disp = 0;
          bridge_loops = 0;
          bridge_tick = BRIDGE_BLINK;
          head_tick = BRIDGE_BLINK;
          head_pos = 0;
          head_cols = 0;
          head_toggle = 0;
          meta_ticker = 0;
          sine_index = 0;
          volume = 1;
          simon_ticks = 0;
          simon_speaks = 0;
          melody_pos = 0;
          melody_ticks = 0;
          motor_mel = 0;

      }

      if (ticket == 0xC1) {
        if ((meta_ctrl & 0x01)==0) {
        badge_motor(TRUE);
        state_ctrl = META_STATE;
        meta_ticker = 0;
        meta_ctrl |= 0x01;
        f_ptr = (unsigned char *)F_TAG;
        Flash_clr(f_ptr);
        WriteFlashByte(f_ptr++, 0x6D);
        WriteFlashByte(f_ptr++, state_ctrl);
        WriteFlashByte(f_ptr++, meta_ctrl);
        }
      } else if (ticket == 0xC2) {
        if ((meta_ctrl & 0x02)==0) {
        badge_motor(TRUE);
        state_ctrl = META_STATE;
        meta_ticker = 0;
        meta_ctrl |= 0x02;
        f_ptr = (unsigned char *)F_TAG;
        Flash_clr(f_ptr);
        WriteFlashByte(f_ptr++, 0x6D);
        WriteFlashByte(f_ptr++, state_ctrl);
        WriteFlashByte(f_ptr++, meta_ctrl);
         }
      } else if (ticket == 0xC3) {
        if ((meta_ctrl & 0x04)==0) {
        badge_motor(TRUE);
        state_ctrl = META_STATE;
        meta_ticker = 0;
        meta_ctrl |= 0x04;
        f_ptr = (unsigned char *)F_TAG;
        Flash_clr(f_ptr);
        WriteFlashByte(f_ptr++, 0x6D);
        WriteFlashByte(f_ptr++, state_ctrl);
        WriteFlashByte(f_ptr++, meta_ctrl);
         }
      }

      if (state_ctrl == META_STATE) {
        if ((meta_ctrl & 0x07) == 0x07) {
            badge_motor(TRUE);
            state_ctrl = BRIDGE_STATE;  
            bridge_pos = 0;          
        f_ptr = (unsigned char *)F_TAG;
        Flash_clr(f_ptr);
        WriteFlashByte(f_ptr++, 0x6D);
        WriteFlashByte(f_ptr++, state_ctrl);
        WriteFlashByte(f_ptr++, meta_ctrl);
         }
      }
      
      if(state_ctrl != FINISH_STATE) { 
      if(state_ctrl != MELODY_STATE) {
        simon_in = 1;
        if(ticket == 0xC4) {
          if(simon_echo!=1){
            badge_motor(TRUE);
            simon_echo = 1;
            state_ctrl = HEAD_STATE;
            simon_tags |= 0x01;
            simon_says[simon_index++] = 1;
            simon_in = 0;
            TBCTL &= ~(MC0);
            TBCTL |= TBCLR;
           TBCCR0 = TONES[0];
            TBCTL |= MC0;
           simon_speaks = 1;
          }
        } else if (ticket == 0xC5) {
          if(simon_echo!=2){
            badge_motor(TRUE);
            simon_echo = 2;
            state_ctrl = HEAD_STATE;
            simon_tags |= 0x02;
            simon_says[simon_index++] = 2;
            simon_in = 0;
            TBCTL &= ~(MC0);
            TBCTL |= TBCLR;
           TBCCR0 = TONES[1];
            TBCTL |= MC0;
           simon_speaks = 1;
          }
        } else if (ticket == 0xC6) {
          if(simon_echo!=3){
            badge_motor(TRUE);
            simon_echo = 3;
            state_ctrl = HEAD_STATE;
            simon_tags |= 0x04;
            simon_says[simon_index++] = 3;
            simon_in = 0;
           TBCTL &= ~(MC0);
            TBCTL |= TBCLR;
             TBCCR0 = TONES[2];
            TBCTL |= MC0;
           simon_speaks = 1;
          }
        } else if (ticket == 0xC7) {
           if(simon_echo!=4){
            badge_motor(TRUE);
            simon_echo = 4;
            state_ctrl = HEAD_STATE;
            simon_tags |= 0x08;
            simon_says[simon_index++] = 4;
            simon_in = 0;
            TBCTL &= ~(MC0);
            TBCTL |= TBCLR;
            TBCCR0 = TONES[3];
            TBCTL |= MC0;
           simon_speaks = 1;
           }
        } else if (ticket == 0xC8) {
          if(simon_echo!=5){
            badge_motor(TRUE);
            simon_echo = 5;
            state_ctrl = HEAD_STATE;
            simon_tags |= 0x10;
            simon_says[simon_index++] = 5;
            simon_in = 0;
            TBCTL &= ~(MC0);
            TBCTL |= TBCLR;
            TBCCR0 = TONES[4];
           TBCTL |= MC0;
            simon_speaks = 1;
           }
        }
      }}
        if (state_ctrl == HEAD_STATE && simon_in == 0) {
          simon_echo_ticks = 0;
          // CHECK MELODY
          if(simon_says[simon_index-1] != SIMON_MELODY[simon_index-1]) {
            simon_index = 0;
            if(simon_once == 1) {
              //PLAY MELODY
            badge_led_grid_draw_screen(0x111,0x0aa,0x044,0x0aa,0x111);
              state_ctrl = MELODY_STATE;
              simon_speaks = 1;
 //             Wait(500);
            } else if ((0x1F&simon_tags) == 0x1F) {
              // PLAY MELODY
            badge_led_grid_draw_screen(0x111,0x0aa,0x044,0x0aa,0x111);
              simon_speaks = 1;
              state_ctrl = MELODY_STATE;
 //             Wait(500);
              simon_tags = 0;
              simon_once = 1;
            }
          } else {
            if (simon_index == 15) {
              state_ctrl = FINISH_STATE;   
              f_ptr = (unsigned char *)F_TAG;
              Flash_clr(f_ptr);
              WriteFlashByte(f_ptr++, 0x6D);
              WriteFlashByte(f_ptr++, state_ctrl);
              WriteFlashByte(f_ptr++, meta_ctrl);
            }
          }
        } 

      if (new_ticket == 0) {
        dir = 0;
        ticker_ir = 0;
        b_led1 = 0;
        b_led2 = 0;
        b_led3 = 0;
        b_led4 = 0;
      }  
        ticker_ir = 0;
        new_ticket = 1;
    }
  }


  if (new_ticket == 1) {
    if (b_led1 <= 0x32){
      badge_single_led(BLED1,b_led1++);
      badge_single_led(BLED2,b_led2++);
      badge_single_led(BLED3,b_led3++);
      badge_single_led(BLED4,b_led4++);
    }        
  }

  if(++ticker_ir == 20) {
      new_ticket = 0;
      dimming_led = 1;
   } 

  if(b_led1 && dimming_led){
      badge_single_led(BLED1,--b_led1);
      badge_single_led(BLED2,--b_led2);
      badge_single_led(BLED3,--b_led3);
      badge_single_led(BLED4,--b_led4);
  } else {
    dimming_led = 0;
  }


  if (state_ctrl == META_STATE) {
    meta_ticker++;
    led1=led2=led3=led4=led5=0;
    if (meta_ctrl & 0x01) {
          led1 += META_1_1;
          led2 += META_1_2;
          led3 += META_1_3;
          led4 += META_1_4;
          led5 += META_1_5;
    }
    if (meta_ctrl & 0x02) {
        led1 += META_2_1;
        led2 += META_2_2;
        led3 += META_2_3;
        led4 += META_2_4;
        led5 += META_2_5;
    }
    if (meta_ctrl & 0x04) {
        led1 += META_3_1;
        led2 += META_3_2;
        led3 += META_3_3;
        led4 += META_3_4;
        led5 += META_3_5;
    }
    badge_led_grid_draw_screen(led1,led2,led3,led4,led5);      
  }

  if (state_ctrl == MELODY_STATE) {
    if (--melody_ticks) {
        melody_ticks = BRIDGE_BLINK;
        if (melody_pos < 15) {
          if(simon_speaks == 0) {
            Wait(200);
            TBCTL &= ~(MC0);
            TBCTL |= TBCLR;
            TBCCR0 = TONES[SIMON_MELODY[melody_pos++]-1];
            TBCTL |= MC0;
            simon_speaks = 1;
          }
        } else {
          melody_pos = 0;
          state_ctrl = HEAD_STATE;
          simon_tags = 0;
          simon_echo = 0;
        }
    }
  }
            
 
  if (state_ctrl == BRIDGE_STATE) {
    if (--bridge_tick) {
        bridge_tick = BRIDGE_BLINK;
        if (bridge_pos < 5) {
          ++bridge_pos;
          if(bridge_toggle == 0) {
            badge_led_grid_draw_screen(0x1ff,0x1ff,0x1ff,0x1ff,0x1ff);
            bridge_toggle = 1;
          } else {
            badge_led_grid_draw_screen(0,0,0,0,0);
            bridge_toggle = 0;
          }
          led1=0;
          led2=0;
          led3=0;
          led4=0;
          led5=0;
          bridge_cols = 0;
          memcpy(bridge_deck,B_ALPHA[BRIDGE_MESSAGE[0]],5);
        } else {
          if (bridge_cols++ < 6) {
            led1 = led1 << 1;
            led2 = led2 << 1;
            led3 = led3 << 1;
            led4 = led4 << 1;
            led5 = led5 << 1;
            if(bridge_deck[0]&0x80) led1 |= 0x01;
            if(bridge_deck[1]&0x80) led2 |= 0x01;
            if(bridge_deck[2]&0x80) led3 |= 0x01;
            if(bridge_deck[3]&0x80) led4 |= 0x01;
            if(bridge_deck[4]&0x80) led5 |= 0x01;
            bridge_deck[0] = bridge_deck[0] << 1;
            bridge_deck[1] = bridge_deck[1] << 1;
            bridge_deck[2] = bridge_deck[2] << 1;
            bridge_deck[3] = bridge_deck[3] << 1;
            bridge_deck[4] = bridge_deck[4] << 1;
            badge_led_grid_draw_screen(led1,led2,led3,led4,led5);
          } else {
            bridge_cols = 0;
            memcpy(bridge_deck,B_ALPHA[BRIDGE_MESSAGE[bridge_pos-4]],5);
            if(++bridge_pos > 35) {
              bridge_pos = 4;
              bridge_loops++;
            }
            bridge_toggle = 1;
          }          
        }
          
    }
  }


  if (state_ctrl == HEAD_STATE) {
    if (--head_tick) {
        head_tick = BRIDGE_BLINK;
        if (head_pos < 5) {
          ++head_pos;
          if(head_toggle == 0) {
            badge_led_grid_draw_screen(0x1ff,0x1ff,0x1ff,0x1ff,0x1ff);
            head_toggle = 1;
          } else {
            badge_led_grid_draw_screen(0,0,0,0,0);
            head_toggle = 0;
          }
          led1=0;
          led2=0;
          led3=0;
          led4=0;
          led5=0;
          head_cols = 0;
          memcpy(head_deck,B_ALPHA[HEAD_MESSAGE[0]],5);
        } else {
          if (head_cols++ < 6) {
            led1 = led1 << 1;
            led2 = led2 << 1;
            led3 = led3 << 1;
            led4 = led4 << 1;
            led5 = led5 << 1;
            if(head_deck[0]&0x80) led1 |= 0x01;
            if(head_deck[1]&0x80) led2 |= 0x01;
            if(head_deck[2]&0x80) led3 |= 0x01;
            if(head_deck[3]&0x80) led4 |= 0x01;
            if(head_deck[4]&0x80) led5 |= 0x01;
            head_deck[0] = head_deck[0] << 1;
            head_deck[1] = head_deck[1] << 1;
            head_deck[2] = head_deck[2] << 1;
            head_deck[3] = head_deck[3] << 1;
            head_deck[4] = head_deck[4] << 1;
            badge_led_grid_draw_screen(led1,led2,led3,led4,led5);
          } else {
            head_cols = 0;
            memcpy(head_deck,B_ALPHA[HEAD_MESSAGE[head_pos-4]],5);
            if(++head_pos > 15) {
              head_pos = 4;
            }
            head_toggle = 1;
          }          
        }
    }
  }

  if (state_ctrl == FINISH_STATE) {
    if (--fin_tick) {
        fin_tick = BRIDGE_BLINK;
        if (fin_pos < 5) {
          ++fin_pos;
          if(fin_toggle == 0) {
            badge_led_grid_draw_screen(0x1ff,0x1ff,0x1ff,0x1ff,0x1ff);
            fin_toggle = 1;
          } else {
            badge_led_grid_draw_screen(0,0,0,0,0);
            fin_toggle = 0;
          }
          led1=0;
          led2=0;
          led3=0;
          led4=0;
          led5=0;
          fin_cols = 0;
          memcpy(fin_deck,B_ALPHA[FINISH_MESSAGE[0]],5);
        } else {
          if (fin_cols++ < 6) {
            led1 = led1 << 1;
            led2 = led2 << 1;
            led3 = led3 << 1;
            led4 = led4 << 1;
            led5 = led5 << 1;
            if(fin_deck[0]&0x80) led1 |= 0x01;
            if(fin_deck[1]&0x80) led2 |= 0x01;
            if(fin_deck[2]&0x80) led3 |= 0x01;
            if(fin_deck[3]&0x80) led4 |= 0x01;
            if(fin_deck[4]&0x80) led5 |= 0x01;
            fin_deck[0] = fin_deck[0] << 1;
            fin_deck[1] = fin_deck[1] << 1;
            fin_deck[2] = fin_deck[2] << 1;
            fin_deck[3] = fin_deck[3] << 1;
            fin_deck[4] = fin_deck[4] << 1;
            badge_led_grid_draw_screen(led1,led2,led3,led4,led5);
          } else {
            fin_cols = 0;
            memcpy(fin_deck,B_ALPHA[FINISH_MESSAGE[fin_pos-4]],5);
            if(++fin_pos > 47) {
              fin_pos = 4;
            }
            fin_toggle = 1;
          }          
        }
    }
  }



  if(state_ctrl == MSG_STATE) {
    if (--msg_tick) {
        msg_tick = BRIDGE_BLINK;
        if (msg_pos > 15){
          if(msg_pos < 20) {
             msg_pos++;
             if(msg_toggle == 0) {
                 badge_led_grid_draw_screen(0x1ff,0x1ff,0x1ff,0x1ff,0x1ff);
                 msg_toggle = 1;
             } else {
                 badge_led_grid_draw_screen(0,0,0,0,0);
                 msg_toggle = 0;
             }
          } else {
             msg_pos = 0;
             state_ctrl = old_state;
             bridge_pos = 4;          
          }
       } else {

        if (msg_pos < 5) {
          ++msg_pos;
          if(msg_toggle == 0) {
            badge_led_grid_draw_screen(0x1ff,0x1ff,0x1ff,0x1ff,0x1ff);
            msg_toggle = 1;
          } else {
            badge_led_grid_draw_screen(0,0,0,0,0);
            msg_toggle = 0;
          }
          led1=0;
          led2=0;
          led3=0;
          led4=0;
          led5=0;
          msg_cols = 0;
          memcpy(msg_deck,B_ALPHA[MSG_MESSAGE[0]],5);
        } else {
          if (msg_cols++ < 6) {
            if(msg_pos < 13){
            led1 = led1 << 1;
            led2 = led2 << 1;
            led3 = led3 << 1;
            led4 = led4 << 1;
            led5 = led5 << 1;
            if(msg_deck[0]&0x80) led1 |= 0x01;
            if(msg_deck[1]&0x80) led2 |= 0x01;
            if(msg_deck[2]&0x80) led3 |= 0x01;
            if(msg_deck[3]&0x80) led4 |= 0x01;
            if(msg_deck[4]&0x80) led5 |= 0x01;
            msg_deck[0] = msg_deck[0] << 1;
            msg_deck[1] = msg_deck[1] << 1;
            msg_deck[2] = msg_deck[2] << 1;
            msg_deck[3] = msg_deck[3] << 1;
            msg_deck[4] = msg_deck[4] << 1;
            badge_led_grid_draw_screen(led1,led2,led3,led4,led5);
            }
          } else {
            msg_cols = 0;
            if(msg_pos == 9) {
              memcpy(msg_deck,B_NUM[digit_10],5);
            } else if (msg_pos == 10) {
              memcpy(msg_deck,B_NUM[digit_1],5);
            } else if (msg_pos == 11) {
              memcpy(msg_deck,B_ALPHA[0],5);
            } else {
              memcpy(msg_deck,B_ALPHA[MSG_MESSAGE[msg_pos-4]],5);
            }
            if(++msg_pos > 12) {
                if(msg_pos < 16) {
                led1=led2=led3=led4=led5=0;
                if (msg_meta & 0x01) {
                  led1 += META_1_1;
                  led2 += META_1_2;
                  led3 += META_1_3;
                  led4 += META_1_4;
                  led5 += META_1_5;
                }
                if (msg_meta & 0x02) {
                  led1 += META_2_1;
                  led2 += META_2_2;
                  led3 += META_2_3;
                  led4 += META_2_4;
                  led5 += META_2_5;
                }
                if (msg_meta & 0x04) {
                  led1 += META_3_1;
                  led2 += META_3_2;
                  led3 += META_3_3;
                  led4 += META_3_4;
                  led5 += META_3_5;
                }
                badge_led_grid_draw_screen(led1,led2,led3,led4,led5); 
                msg_toggle = 0;
                }
            }
          }          
        }
      }    
    }

  }

  }
}

void usart1_rx (void) __interrupt[UART1RX_VECTOR] 
{
  unsigned char ret_val;
  unsigned char status;
  unsigned char newPacket[DATA_BUFFER_LENGTH];
  char tmp_buf[2];

  ret_val = handle_rf_byte(RXBUF1, &status, newPacket);
  if(ret_val == NEW_STATUS) {
    if ( status & 0x01) {
      Wait(50);
      get_rf_packet();
    } 
  } else if (ret_val == NEW_RX_PACKET) {
    // RECEIVED NEW PACKET IN BUFFER
    _NOP();
    if((state_ctrl == META_STATE && meta_ticker > 200) || (state_ctrl == BRIDGE_STATE && bridge_loops > 2)) {
      if (newPacket[1] & 0x80) {
        if(team_new[0x3F&newPacket[1]] == 0) {
          team_new[0x3F&newPacket[1]] = 1;
          team_ticks[0x3F&newPacket[1]] = 0;
          msg_meta = newPacket[4];
          itoa(0x3f&newPacket[1],tmp_buf,10);
          if((0x3f&newPacket[1]) > 10) {
            digit_1 = tmp_buf[0] - 48;
            digit_10 = tmp_buf[1] - 48;
          }else{
            digit_1 = tmp_buf[0] - 48;
            digit_10 = 0;
          }
          old_state = state_ctrl;
          state_ctrl = MSG_STATE;
        }
      }
    }
  }
}


// Timer_A3 Interrupt Vector (TAIV) handler
void Timer_A(void) __interrupt [TIMERA1_VECTOR] 
{
 switch( TAIV )
 {
  unsigned char q;

   case  2: break;                      // CCR1 not used
   case  4: break;                      // CCR2 not used
   case 10: 
    if (++tmr_counter == TIMER_SCALER) {
      tmr_counter = 0;
      if (comm_toggle) {
        pRFData[0] = meta_ctrl;
       send_rf_packet(0x00,0xFF,0xFF,0x01,pRFData); 
        comm_toggle = 0;
      } else {
        badge_iru_send_ir_byte(MY_BADGE_ID);
       comm_toggle = 1;
      }

      for (q=0;q<50;q++) {
        if(team_new[q]==1){
          if(++team_ticks[q] == TEAM_TIMEOUT) {
            team_new[q] = 0;
          }
        }
      }
    }  
        break;
 }
}

void Timer_B(void) __interrupt [TIMERB0_VECTOR]
{
	int tmp_1, tmp_2;
        if(simon_echo_ticks++ > 63000 ) {
          simon_echo_ticks = 0;
          simon_echo = 0;
        }
        if(simon_speaks == 1) {
          tmp_1 = tmp_2 = TRIANGLE_WAVE[sine_index++]*volume;
          tmp_1 &= 0x003f;
          tmp_1 += 23;
          tmp_2 = tmp_2 >> 6;
          tmp_2 &= 0x00ff;
          tmp_2 += 23;
          TBCCR2 = tmp_2;
          TBCCR1 = tmp_1;
          if(sine_index==81) {
            sine_index = 0;
            if(++simon_ticks > 120) {
              simon_ticks = 0;
              simon_speaks = 0;
            }
          }
        } else {
          tmp_1 = tmp_2 = TRIANGLE_WAVE[0]*volume;
          tmp_1 &= 0x003f;
          tmp_1 += 23;
          tmp_2 = tmp_2 >> 6;
          tmp_2 &= 0x00ff;
          tmp_2 += 23;
          TBCCR2 = tmp_2;
          TBCCR1 = tmp_1;
          TBCCR0 = 0xff;
 
        }	
} 

bool WriteFlashByte(unsigned char *Addr, unsigned char Data) {
  unsigned char IE1Image;

  _DINT();
  while(FCTL3&BUSY);
  // Disable NMI, Access Violation & Osc. Fault interrupts.
  IE1Image = IE1;
  IE1 &= ~(NMIIE+ACCVIE+OFIE);
  FCTL3 = FWKEY; // Clear lock bit
//  FCTL2 = FWKEY+FSSEL_2+11;
  FCTL1 = FWKEY+WRT; // Enable write to flash
  *Addr = Data; // Write the word to flash
  FCTL1 = FWKEY; // Reset write bit
  FCTL3 = FWKEY+LOCK; // Set lock bit
  IE1 = IE1Image;
  while(FCTL3&BUSY);
  _EINT();
  return *Addr==Data;
}

void Flash_clr( unsigned char *Data_ptr )
{
  unsigned char IE1Image;

  _DINT();
  while(FCTL3&BUSY);
  // Disable NMI, Access Violation & Osc. Fault interrupts.
  IE1Image = IE1;
  IE1 &= ~(NMIIE+ACCVIE+OFIE);
  FCTL3 = 0x0A500;          /* Lock = 0 */
  FCTL1 = 0x0A502;          /* ERASE = 1 */
  *Data_ptr=0;              /* erase Flash segment */
  FCTL1 = 0x0A500;          /* ERASE = 0 */
  FCTL3 = 0x0A510;          /* Lock = 1 */
  IE1 = IE1Image;
  while(FCTL3&BUSY);
  _EINT();
}

