#include  "badge_alpha.h"

#include  "badge_init.h"
#include  "badge_isr.h"
#include  "badge_leds.h"
#include  "badge_basics.h"
#include  "badge_flash.h"
#include  "badge_lcd.h"
#include  "badge_iru.h"
#include  "badge_rf.h"

#include <stdlib.h>

#define   MY_BADGE_ID    0x86
#define   TIMER_SCALER  75
#define   BRIDGE_BLINK  60
#define   TEAM_TIMEOUT  500
#define   IR_TIMEOUT    3

#define   META_1_1    0x1F0
#define   META_1_2    0x1E0
#define   META_1_3    0x1C0
#define   META_1_4    0x180
#define   META_1_5    0x100
#define   META_2_1    0x00E
#define   META_2_2    0x01C
#define   META_2_3    0x038
#define   META_2_4    0x070
#define   META_2_5    0x0E0
#define   META_3_1    0x001
#define   META_3_2    0x003
#define   META_3_3    0x007
#define   META_3_4    0x00F
#define   META_3_5    0x01F

#define   META_STATE    0
#define   BRIDGE_STATE  1
#define   MSG_STATE     2
#define   MELODY_STATE  3

#define   F_TAG       0x01000     /* Flash address for state */
#define   F_STATE     0x01001     /* Flash address for state */
#define   F_META      0x01002     /* Flash address for Meta */

unsigned int tmr_counter;
unsigned char comm_toggle;
unsigned char pRFData[5];
unsigned int led1,led2,led3,led4,led5;
unsigned char bridge_disp;
unsigned char bridge_toggle, bridge_tick, bridge_pos, bridge_cols;
unsigned int bridge_loops;
unsigned char bridge_deck[5];
unsigned char msg_toggle, msg_tick, msg_pos, msg_cols, msg_name,msg_loops;
unsigned char msg_deck[5];

unsigned char state_ctrl;
unsigned char meta_ctrl;

unsigned long team_ticks[50];
unsigned char team_new[50];
unsigned char old_state;
unsigned char digit_10, digit_1,msg_meta;

unsigned char simon_tags;
unsigned char simon_says[16];
unsigned char simon_index;
unsigned char simon_in;
unsigned char simon_echo;
unsigned char simon_once;

unsigned char head_deck[5];
unsigned char head_toggle, head_tick, head_pos, head_cols;
unsigned char fin_deck[5];
unsigned char fin_toggle, fin_tick, fin_pos, fin_cols;

unsigned int sine_index;
unsigned char volume;
unsigned char simon_speaks;
unsigned int simon_ticks;
unsigned long simon_echo_ticks;
unsigned int melody_ticks;
unsigned char melody_pos;
unsigned char motor_mel;

unsigned char *f_ptr;

unsigned long meta_ticker;

unsigned char ir_packet[4];
unsigned char ir_index,ir_echo,ir_counter;

const unsigned char SIMON_MELODY[] = {3,3,4,5,5,4,3,2,1,1,2,3,2,1,1};
const unsigned int TONES[] = {(CLOCK_RATE/22)-1,(CLOCK_RATE/25)-1,(CLOCK_RATE/28)-1,(CLOCK_RATE/30)-1,(CLOCK_RATE/33)-1};
// _ABCDEFGHIJKLMNOPQRSTUVWXYZ
// 012345678901234567890123456
const unsigned char BRIDGE_MESSAGE[] = {18,5,20,21,18,14,0,20,15,0,2,1,18,20,15,19,0,0};
const unsigned char JOE_MSG[] = {8,9,0,10,15,5,0,0};
const unsigned char MAT_MSG[] = {8,9,0,13,1,20,0,0};
const unsigned char JEN_MSG[] = {8,9,0,10,5,14,0,0};

bool WriteFlashByte(unsigned char *Addr, unsigned char Data);
void Flash_clr( unsigned char *Data_ptr );

void main(void)
{

  unsigned short inte = 0;
  unsigned short dir = 0;
  unsigned char ticker,ticket,new_ticket,ticker_ir;
  unsigned char b_led1,b_led2,b_led3,b_led4;
  unsigned int i;
  unsigned char pid, dimming_led; 

  
  tmr_counter = 0;
  comm_toggle = 0;
  bridge_toggle = 0;
  bridge_disp = 0;
  bridge_loops = 0;
  bridge_tick = BRIDGE_BLINK;
  head_tick = BRIDGE_BLINK;
  head_pos = 0;
  head_cols = 0;
  head_toggle = 0;
  meta_ticker = 0;
  sine_index = 0;
  volume = 1;
  simon_ticks = 0;
  simon_speaks = 0;
  melody_pos = 0;
  melody_ticks = 0;
  motor_mel = 0;
  ir_index = 0;
  ir_echo = 0;
  ir_counter = 0;
  msg_loops = 0;

  b_led1 = 0;
  b_led2 = 0;
  b_led3 = 0;
  b_led4 = 0;

  badge_init_osc();
  badge_init_io();
  badge_init_spi();
  badge_init_uart();
  badge_init_pwm();

   state_ctrl = META_STATE;
   meta_ctrl = 0;



  Wait(500);                       // Power Up Wait

  badge_led_grid_init();
  badge_led_grid_draw_screen(0,0,0,0,0);

  _EINT();

  set_rf_settings(0x00,MY_BADGE_ID,0x10,0x03);
  Wait(250);
  set_rf_settings(0x00,MY_BADGE_ID,0x10,0x03);
  Wait(250);

  TACTL = TASSEL1 + TACLR + TAIE;       // SMCLK, clr. TAR, interrupt
  TACTL |= MC1;                         // Start Timer_a in continous

 
  while(TRUE) {


    Wait(100);
    
    if (state_ctrl != MELODY_STATE) {
      badge_motor(FALSE);
    } else {
      if(++motor_mel > 10) {
      motor_mel = 0;
      badge_motor(FALSE);
      }
    }

    get_rf_status();

  if (badge_poll_switch() == SWITCH_UP){
    volume = 2;
    badge_iru_send_ir_byte(0xFF);
    badge_iru_send_ir_byte(0xC1);
    badge_iru_send_ir_byte(0xC1);
    badge_iru_send_ir_byte(0x6D);
    while (badge_poll_switch() == SWITCH_UP);
  } else if (badge_poll_switch() == SWITCH_DOWN) {
    volume = 1;
    badge_iru_send_ir_byte(0xFF);
    badge_iru_send_ir_byte(0xC2);
    badge_iru_send_ir_byte(0xC2);
    badge_iru_send_ir_byte(0x6D);
    while (badge_poll_switch() == SWITCH_DOWN);
  } else if (badge_poll_switch() == SWITCH_IN) {
    pRFData[0] = 0xD1;
    send_rf_packet(0x00,0xFF,0xFF,0x01,pRFData); 
    while (badge_poll_switch() == SWITCH_IN);
  }



  ticket = badge_iru_status();
  if (ticket & UART_RX_STATUS) {
    // UART BYTE RECEIVED HANDLE
  } else if (ticket & IR_RX_STATUS) {
    // IR BYTE RECEIVED HANDLE
    ticket = badge_iru_get_ir_byte();
    if(ir_index == 0) {
      if(ticket == 0xFF) {
        ir_index = 1;
        ir_packet[0] = ticket;
        ir_counter = 0;
      }
    } else {
      ir_packet[ir_index++] = ticket;
      ir_counter = 0;
      if(ir_index==4){
        // IR PACKET RECEIVED
        ir_index = 0;
        if(ir_packet[1] == ir_packet[2]) {
          if(ir_packet[3] == 0x6D) {
            //VALID PACKET RECEIVED
            ticket = ir_packet[1];
            if(ticket != ir_echo) {
              ir_echo = ticket;
              if (ticket != MY_BADGE_ID) {
                if (ticket == 0xC1) {
                  if ((meta_ctrl & 0x01)==0) {
                    badge_motor(TRUE);
                    state_ctrl = META_STATE;
                    meta_ticker = 0;
                    meta_ctrl |= 0x01;
                  }
                } else if (ticket == 0xC2) {
                  if ((meta_ctrl & 0x02)==0) {
                    badge_motor(TRUE);
                    state_ctrl = META_STATE;
                    meta_ticker = 0;
                    meta_ctrl |= 0x02;
                  }
                } else if (ticket == 0xC3) {
                  if ((meta_ctrl & 0x04)==0) {
                    badge_motor(TRUE);
                    state_ctrl = META_STATE;
                    meta_ticker = 0;
                    meta_ctrl |= 0x04;
                  }
                } else if (ticket == 0x86) {
                    badge_motor(TRUE);
                    state_ctrl = MSG_STATE;
                    msg_name = 0;
                } else if (ticket == 0x89) {
                    badge_motor(TRUE);
                    state_ctrl = MSG_STATE;
                    msg_name = 1;
                } else if (ticket == 0x8C) {
                    badge_motor(TRUE);
                    state_ctrl = MSG_STATE;
                    msg_name = 2;
                }
  
                if (state_ctrl == META_STATE) {
                  if ((meta_ctrl & 0x07) == 0x07) {
                    badge_motor(TRUE);
                    state_ctrl = MELODY_STATE;  
                    badge_led_grid_draw_screen(0x111,0x0aa,0x044,0x0aa,0x111);
                    simon_speaks = 1;
                    melody_pos = 0;
                  }
                }
   
                if (new_ticket == 0) {
                  dir = 0;
                  ticker_ir = 0;
                  b_led1 = 0;
                  b_led2 = 0;
                  b_led3 = 0;
                  b_led4 = 0;
                }  
                ticker_ir = 0;
                new_ticket = 1;
              
              }
            }
          }
        }
      }
    }
  }

  if (new_ticket == 1) {
    if (b_led1 <= 0x32){
      badge_single_led(BLED1,b_led1++);
      badge_single_led(BLED2,b_led2++);
      badge_single_led(BLED3,b_led3++);
      badge_single_led(BLED4,b_led4++);
    }        
  }

  if(++ticker_ir == 20) {
      new_ticket = 0;
      dimming_led = 1;
   } 

  if(b_led1 && dimming_led){
      badge_single_led(BLED1,--b_led1);
      badge_single_led(BLED2,--b_led2);
      badge_single_led(BLED3,--b_led3);
      badge_single_led(BLED4,--b_led4);
  } else {
    dimming_led = 0;
  }


  if (state_ctrl == META_STATE) {
    meta_ticker++;
    led1=led2=led3=led4=led5=0;
    if (meta_ctrl & 0x01) {
          led1 += META_1_1;
          led2 += META_1_2;
          led3 += META_1_3;
          led4 += META_1_4;
          led5 += META_1_5;
    }
    if (meta_ctrl & 0x02) {
        led1 += META_2_1;
        led2 += META_2_2;
        led3 += META_2_3;
        led4 += META_2_4;
        led5 += META_2_5;
    }
    if (meta_ctrl & 0x04) {
        led1 += META_3_1;
        led2 += META_3_2;
        led3 += META_3_3;
        led4 += META_3_4;
        led5 += META_3_5;
    }
    badge_led_grid_draw_screen(led1,led2,led3,led4,led5);      
  }

  if (state_ctrl == MELODY_STATE) {
    if (--melody_ticks) {
        melody_ticks = BRIDGE_BLINK;
        if (melody_pos < 15) {
          if(simon_speaks == 0) {
            Wait(200);
            TBCTL &= ~(MC0);
            TBCTL |= TBCLR;
            TBCCR0 = TONES[SIMON_MELODY[melody_pos++]-1];
            TBCTL |= MC0;
            simon_speaks = 1;
          }
        } else {
          melody_pos = 0;
          state_ctrl = META_STATE;
          meta_ctrl = 0;
          simon_tags = 0;
          simon_echo = 0;
        }
    }
  }
            
 
  if (state_ctrl == BRIDGE_STATE) {
    if (--bridge_tick) {
        bridge_tick = BRIDGE_BLINK;
        if (bridge_pos < 5) {
          ++bridge_pos;
          if(bridge_toggle == 0) {
            badge_led_grid_draw_screen(0x1ff,0x1ff,0x1ff,0x1ff,0x1ff);
            bridge_toggle = 1;
          } else {
            badge_led_grid_draw_screen(0,0,0,0,0);
            bridge_toggle = 0;
          }
          led1=0;
          led2=0;
          led3=0;
          led4=0;
          led5=0;
          bridge_cols = 0;
          memcpy(bridge_deck,B_ALPHA[BRIDGE_MESSAGE[0]],5);
        } else {
          if (bridge_cols++ < 6) {
            led1 = led1 << 1;
            led2 = led2 << 1;
            led3 = led3 << 1;
            led4 = led4 << 1;
            led5 = led5 << 1;
            if(bridge_deck[0]&0x80) led1 |= 0x01;
            if(bridge_deck[1]&0x80) led2 |= 0x01;
            if(bridge_deck[2]&0x80) led3 |= 0x01;
            if(bridge_deck[3]&0x80) led4 |= 0x01;
            if(bridge_deck[4]&0x80) led5 |= 0x01;
            bridge_deck[0] = bridge_deck[0] << 1;
            bridge_deck[1] = bridge_deck[1] << 1;
            bridge_deck[2] = bridge_deck[2] << 1;
            bridge_deck[3] = bridge_deck[3] << 1;
            bridge_deck[4] = bridge_deck[4] << 1;
            badge_led_grid_draw_screen(led1,led2,led3,led4,led5);
          } else {
            bridge_cols = 0;
            memcpy(bridge_deck,B_ALPHA[BRIDGE_MESSAGE[bridge_pos-4]],5);
            if(++bridge_pos > 21) {
              bridge_pos = 4;
              if(bridge_loops++>2){
                bridge_loops = 0;
                state_ctrl = META_STATE;
              }
            }
            bridge_toggle = 1;
          }          
        }
          
    }
  }

  if (state_ctrl == MSG_STATE) {
    if (--msg_tick) {
        msg_tick = BRIDGE_BLINK;
        if (msg_pos < 5) {
          ++msg_pos;
          if(msg_toggle == 0) {
            badge_led_grid_draw_screen(0x1ff,0x1ff,0x1ff,0x1ff,0x1ff);
            msg_toggle = 1;
          } else {
            badge_led_grid_draw_screen(0,0,0,0,0);
            msg_toggle = 0;
          }
          led1=0;
          led2=0;
          led3=0;
          led4=0;
          led5=0;
          msg_cols = 0;
          if(msg_name == 0){
            memcpy(msg_deck,B_ALPHA[JOE_MSG[0]],5);
          } else if (msg_name == 1) {
            memcpy(msg_deck,B_ALPHA[MAT_MSG[0]],5);
          } else {
            memcpy(msg_deck,B_ALPHA[JEN_MSG[0]],5);
          }
        } else {
          if (msg_cols++ < 6) {
            led1 = led1 << 1;
            led2 = led2 << 1;
            led3 = led3 << 1;
            led4 = led4 << 1;
            led5 = led5 << 1;
            if(msg_deck[0]&0x80) led1 |= 0x01;
            if(msg_deck[1]&0x80) led2 |= 0x01;
            if(msg_deck[2]&0x80) led3 |= 0x01;
            if(msg_deck[3]&0x80) led4 |= 0x01;
            if(msg_deck[4]&0x80) led5 |= 0x01;
            msg_deck[0] = msg_deck[0] << 1;
            msg_deck[1] = msg_deck[1] << 1;
            msg_deck[2] = msg_deck[2] << 1;
            msg_deck[3] = msg_deck[3] << 1;
            msg_deck[4] = msg_deck[4] << 1;
            badge_led_grid_draw_screen(led1,led2,led3,led4,led5);
          } else {
            msg_cols = 0;
            if(msg_name == 0) {
              memcpy(msg_deck,B_ALPHA[JOE_MSG[msg_pos-4]],5);
            } else if (msg_name == 1) {
              memcpy(msg_deck,B_ALPHA[MAT_MSG[msg_pos-4]],5);
            } else {
              memcpy(msg_deck,B_ALPHA[JEN_MSG[msg_pos-4]],5);
            }              
            if(++msg_pos > 11) {
              msg_pos = 4;
              if(msg_loops++>2){
                msg_loops = 0;
                state_ctrl = META_STATE;
              }
           }
            msg_toggle = 1;
          }          
        }
          
    }
  }



  }
}

void usart1_rx (void) __interrupt[UART1RX_VECTOR] 
{
  unsigned char ret_val;
  unsigned char status;
  unsigned char newPacket[DATA_BUFFER_LENGTH];
  char tmp_buf[2];

  ret_val = handle_rf_byte(RXBUF1, &status, newPacket);
  if(ret_val == NEW_STATUS) {
    if ( status & 0x01) {
      Wait(50);
      get_rf_packet();
    } 
  } else if (ret_val == NEW_RX_PACKET) {
    // RECEIVED NEW PACKET IN BUFFER
    _NOP();
    if(newPacket[4] == 0xD1) {
      state_ctrl = BRIDGE_STATE;
      bridge_pos = 0;          
      simon_echo = 0;
    }
  }
}


// Timer_A3 Interrupt Vector (TAIV) handler
void Timer_A(void) __interrupt [TIMERA1_VECTOR] 
{
 switch( TAIV )
 {
  unsigned char q;

   case  2: break;                      // CCR1 not used
   case  4: break;                      // CCR2 not used
   case 10: 
    if (++tmr_counter == TIMER_SCALER) {
      tmr_counter = 0;
       badge_iru_send_ir_byte(0xFF);
       Wait(1);
       badge_iru_send_ir_byte(MY_BADGE_ID);
       Wait(1);
       badge_iru_send_ir_byte(MY_BADGE_ID);
       Wait(1);
       badge_iru_send_ir_byte(0x6D);

        if(simon_echo_ticks++ > 30 ) {
          simon_echo_ticks = 0;
          ir_echo = 0;
        }

      if(++ir_counter == IR_TIMEOUT) {
        ir_index = 0;
      }
    }  
        break;
 }
}

void Timer_B(void) __interrupt [TIMERB0_VECTOR]
{
	int tmp_1, tmp_2;
        if(simon_speaks == 1) {
          tmp_1 = tmp_2 = TRIANGLE_WAVE[sine_index++]*volume;
          tmp_1 &= 0x003f;
          tmp_1 += 23;
          tmp_2 = tmp_2 >> 6;
          tmp_2 &= 0x00ff;
          tmp_2 += 23;
          TBCCR2 = tmp_2;
          TBCCR1 = tmp_1;
          if(sine_index==81) {
            sine_index = 0;
            if(++simon_ticks > 120) {
              simon_ticks = 0;
              simon_speaks = 0;
            }
          }
        } else {
          tmp_1 = tmp_2 = TRIANGLE_WAVE[0]*volume;
          tmp_1 &= 0x003f;
          tmp_1 += 23;
          tmp_2 = tmp_2 >> 6;
          tmp_2 &= 0x00ff;
          tmp_2 += 23;
          TBCCR2 = tmp_2;
          TBCCR1 = tmp_1;
          TBCCR0 = 0xff;
 
        }	
} 

bool WriteFlashByte(unsigned char *Addr, unsigned char Data) {
  unsigned char IE1Image;

  _DINT();
  while(FCTL3&BUSY);
  // Disable NMI, Access Violation & Osc. Fault interrupts.
  IE1Image = IE1;
  IE1 &= ~(NMIIE+ACCVIE+OFIE);
  FCTL3 = FWKEY; // Clear lock bit
//  FCTL2 = FWKEY+FSSEL_2+11;
  FCTL1 = FWKEY+WRT; // Enable write to flash
  *Addr = Data; // Write the word to flash
  FCTL1 = FWKEY; // Reset write bit
  FCTL3 = FWKEY+LOCK; // Set lock bit
  IE1 = IE1Image;
  while(FCTL3&BUSY);
  _EINT();
  return *Addr==Data;
}

void Flash_clr( unsigned char *Data_ptr )
{
  unsigned char IE1Image;

  _DINT();
  while(FCTL3&BUSY);
  // Disable NMI, Access Violation & Osc. Fault interrupts.
  IE1Image = IE1;
  IE1 &= ~(NMIIE+ACCVIE+OFIE);
  FCTL3 = 0x0A500;          /* Lock = 0 */
  FCTL1 = 0x0A502;          /* ERASE = 1 */
  *Data_ptr=0;              /* erase Flash segment */
  FCTL1 = 0x0A500;          /* ERASE = 0 */
  FCTL3 = 0x0A510;          /* Lock = 1 */
  IE1 = IE1Image;
  while(FCTL3&BUSY);
  _EINT();
}

